! ***********************************************************************
!
!   Copyright (C) 2010  Bill Paxton
!
!   this file is part of mesa.
!
!   mesa is free software; you can redistribute it and/or modify
!   it under the terms of the gnu general library public license as published
!   by the free software foundation; either version 2 of the license, or
!   (at your option) any later version.
!
!   mesa is distributed in the hope that it will be useful, 
!   but without any warranty; without even the implied warranty of
!   merchantability or fitness for a particular purpose.  see the
!   gnu library general public license for more details.
!
!   you should have received a copy of the gnu library general public license
!   along with this software; if not, write to the free software
!   foundation, inc., 59 temple place, suite 330, boston, ma 02111-1307 usa
!
! ***********************************************************************
 
      module run_star_extras

      use star_lib
      use star_def
      use const_def
      use crlibm_lib
      
      implicit none


      ! these routines are called by the standard run_star check_model
      contains
      
      !include 'standard_run_star_extras.inc'

      subroutine extras_controls(id, ierr)
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
       
         s% extras_startup => extras_startup
         s% extras_check_model => extras_check_model
         s% extras_finish_step => extras_finish_step
         s% extras_after_evolve => extras_after_evolve
         s% how_many_extra_history_columns => how_many_extra_history_columns
         s% data_for_extra_history_columns => data_for_extra_history_columns
         s% how_many_extra_profile_columns => how_many_extra_profile_columns
         s% data_for_extra_profile_columns => data_for_extra_profile_columns  
         s% job% warn_run_star_extras =.false.      


	 !New routines to add

	 s% other_energy => wave_energy
	 !s% other_torque => wave_torque


            
      end subroutine extras_controls




      subroutine wave_energy(id, ierr)
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         integer :: k,nz,k_max,j
	 real(dp) :: conv_freq,heatfrac,L_heat,M_heat
	 real(dp) :: conv_freq_max,L_waves_max,r_max,rho_max,L_conv_max
	 real(dp) :: om2,lambl1,lambl2,lambl3,dr,krl1,krl2,krl3,intkrl1,intkrl2,intkrl3,Tl1,Tl2,Tl3,vg,tprop
	 real(dp) :: wave_break_flag,L_waves,L_max,r_dep,m_dep
	 real(dp) :: diff,diff_t,M_damp,brunt,vel
	 real(dp) :: f_damp_int,f_damp,gamnu,f_fac,x_fac
	 real(dp) :: maxbrunt,k_min,heatsmooth
	 real(dp) :: Tl1b,intkrl1b,conv_edge_flag,intkrl1min,Tl1min,minflag,intmin,krl1min,kmin,bruntmin
         ierr = 0
         call star_ptr(id,s,ierr)
         if (ierr /= 0) return


	 !Calculate maximum wave frequency

	  conv_freq_max=1d-99
	  L_waves_max = 1d-99
	  r_max=1d-99
	  rho_max = 1d-99
	  om2 = 1d-3
	  k_max = s% nz
	  do k=1,s% nz
		conv_freq = s% conv_vel(k)/(4.*s% scale_height(k))
		if ( (s% m(k)/Msun > 0.1) .and. (s% m(k)/Msun < 3.) ) then
			if (conv_freq > conv_freq_max) then
				L_conv_max = s% L_conv(k)
				conv_freq_max = conv_freq !Units of s^{-1}
				L_waves_max = (s% conv_vel(k)/s% csound(k))*s% L_conv(k)
				r_max = s% r(k) !Units of cm
				rho_max = s% rho(k) !Units of g/cm^3
				k_max = k
			end if
		end if
	  end do
	  om2 = pow2(2.*pi*conv_freq_max)

	  heatfrac = 1d-99
	  L_waves = 1d-99
	  M_heat = 1d-99
	  wave_break_flag = 0.
	  r_dep = 1d-99
	  m_dep = 1d-99
	  maxbrunt = 1d99
	  s% extra_heat(:) = 1d-99

	  intkrl1 = 0.
	  intkrl2 = 0.
	  intkrl3 = 0.
	  intmin = 0.
	  intkrl1min = 0.
	  krl1min = 0.

	  !Calculate transmission coefficient
	  do k=1,k_max

	  	minflag = 0.

                vel = 0.
                if (abs(s% v(k)) < 1d11) vel = abs(s% v(k)) !Required if v turned off
                if ( (s% tau(k) > 1d4) .and. (vel < s% csound(k)) ) then !Avoid outflowing surface regions

			lambl1 = 2.*pow2(s% csound(k)/(s% r(k)))
			lambl2 = 6.*pow2(s% csound(k)/(s% r(k)))
			lambl3 = 12.*pow2(s% csound(k)/(s% r(k)))

			if (s% brunt_N2(k) < 0.) brunt = 0. !Avoid artificial drops in Brunt
			if (s% brunt_N2(k) > 0.) brunt = s% brunt_N2(k) !Avoid artificial drops in Brunt

			krl1 = (brunt - om2)*(lambl1 - om2)/(om2*pow2(s% csound(k)))
			krl2 = (brunt - om2)*(lambl2 - om2)/(om2*pow2(s% csound(k)))
			krl3 = (brunt - om2)*(lambl3 - om2)/(om2*pow2(s% csound(k)))

                       	dr = s% dm(k)/(4.*pi*s% rho(k)*pow2(s% r(k)))

			if (krl1 < 0.) then
				intkrl1 = intkrl1 + dr*sqrt(-krl1)
			end if
			if (krl2 < 0.) then
				intkrl2 = intkrl2 + dr*sqrt(-krl2)
			end if
			if (krl3 < 0.) then
				intkrl3 = intkrl3 + dr*sqrt(-krl3)
			end if

			!Find smallest transmission coefficient

			if (krl1 < 0.) then
				minflag = 1.
				intmin = intmin + dr*sqrt(-krl1)
				!if (dr*sqrt(-krl1) > krl1min) then
				!	krl1min = dr*sqrt(-krl1)
				!	kmin = k
				!	bruntmin = brunt
				!endif
			end if

			if (minflag < 0.5) then
				if (intmin > intkrl1min) then
					intkrl1min = intmin
				end if
				intmin = 0.
			end if
		end if
	  end do

	  Tl1 = exp_cr(-2.*intkrl1)
	  Tl2 = exp_cr(-2.*intkrl2)
	  Tl3 = exp_cr(-2.*intkrl3)
	  Tl1min = exp_cr(-2.*intkrl1min)

	  !Calculate evanescent region associated with convective excitation shell
	  intkrl1b = 0.
	  Tl1b = 0.

	  if (s% mass_conv_core < 3d-1) then !Limit to cases without convective core

	  	conv_edge_flag = 0.
	  	do k=k_max,1,-1
			!Integrate from wave excitation point outward to edge of convective shell
			if (s% conv_vel(k) < 1d3 ) then
				conv_edge_flag = 1.
			endif
			if (conv_edge_flag < 0.5) then
				lambl1 = 2.*pow2(s% csound(k)/(s% r(k)))
				if (s% brunt_N2(k) < 0.) brunt = 0. 
				if (s% brunt_N2(k) > 0.) brunt = s% brunt_N2(k) 
				krl1 = (brunt - om2)*(lambl1 - om2)/(om2*pow2(s% csound(k)))
				dr = s% dm(k)/(4.*pi*s% rho(k)*pow2(s% r(k)))
				if (krl1 < 0.) then
					intkrl1b = intkrl1b + dr*sqrt(-krl1)
				end if
			endif
	 	 end do

	  	conv_edge_flag = 0.	  
	  	do k=k_max,s% nz-30
			!Integrate from wave excitation point outward to edge of convective shell
			if (s% conv_vel(k) < 1d3 ) then
				conv_edge_flag = 1.
			endif
			if (conv_edge_flag < 0.5) then
				lambl1 = 2.*pow2(s% csound(k)/(s% r(k)))
				if (s% brunt_N2(k) < 0.) brunt = 0. 
				if (s% brunt_N2(k) > 0.) brunt = s% brunt_N2(k) 
				krl1 = (brunt - om2)*(lambl1 - om2)/(om2*pow2(s% csound(k)))
				dr = s% dm(k)/(4.*pi*s% rho(k)*pow2(s% r(k)))
				if (krl1 < 0.) then
					intkrl1b = intkrl1b + dr*sqrt(-krl1)
				end if
			endif
	  	end do
	  	Tl1b = exp_cr(-2.*intkrl1b) !Limit to cases without convective core
	  end if

	  tprop = 0.
	  f_damp_int = 0.
	  x_fac = 0.

	  !Calculate wave propagation time to base of deposition region
	  do k=k_max,1,-1
		lambl1 = 2.*pow2(s% csound(k)/(s% r(k)))
		if (s% brunt_N2(k) < 0.) brunt = 0. 
		if (s% brunt_N2(k) > 0.) brunt = s% brunt_N2(k) 
		krl1 = (brunt - om2)*(lambl1 - om2)/(om2*pow2(s% csound(k)))
		dr = s% dm(k)/(4.*pi*s% rho(k)*pow2(s% r(k)))
		if (krl1 > 0.) then
			if (om2 > brunt) then
				vg = s% csound(k)
				tprop = tprop + dr/vg
			endif
			if (om2 < brunt) then
				vg = om2*s% r(k)/sqrt(2.*brunt)
				tprop = tprop + dr/vg
				gamnu = -pow2(s% grav(k)*s% gamma1(k)*s% grada(k))
				gamnu = gamnu*(s% d_epsnuc_dlnT(k) - s% d_nonnucneu_dlnT(k))/(brunt*pow2(s% csound(k)*s% csound(k)))
				if (gamnu>0.) then
					f_damp_int = f_damp_int + 2.*gamnu*dr/vg
				end if
			end if
		end if
	  end do
	  f_damp = exp_cr(f_damp_int)
	  f_fac = 1.-1./f_damp
	  x_fac = f_damp_int

	  !heatfrac = Tl1
	  !heatfrac = 0.
	  !heatfrac = Tl1/(x_fac+Tl1)
	  heatfrac = (1.)/(1. + (Tl1b+x_fac)/Tl1min)

	  L_waves = heatfrac*L_waves_max

	  diff = 1d-99
	  diff_t = 1d-99
	  M_damp = 1d99
	  L_heat = 1d-99

	  !Deposit energy
	  do k=k_max,1,-1

		!Deposit Wave Energy
		L_max = 4.*pi*pow2(s% r(k))*s% rho(k)*pow3(s% csound(k))

		!Deposit energy via radiative diffusion
		!if (L_waves < L_max) then
    			diff_t = (16.*boltz_sigma*pow3(s% T(k)))/(3.*pow2(s% rho(k))*s% cp(k)*s% opacity(k))
    			diff = diff_t
			M_damp = 4.*pi*s% rho(k)*pow2(s% r(k))*pow3(s% csound(k))/(om2*diff)
			if (L_waves/Lsun > 1d-1) then
				s% extra_heat(k) = L_waves/M_damp
				L_heat = L_heat + s% extra_heat(k)*s% dm(k)
				L_waves = L_waves - s% extra_heat(k)*s% dm(k)
			end if
		!end if
!		!Deposit according to breaking criteria
		if (L_waves > L_max) then
!			s% extra_heat(k) = (L_waves-L_max)/(s% dm(k))
!			L_waves = L_max
!			!L_waves = L_waves/(1.+2d-6/(1.-s% q(k)))
!			!if (L_heat/Lsun > 1d7) then
!			!	s% extra_heat(k) = 0.
!			!	L_waves = 0.
!			!end if
!			L_heat = L_heat + s% extra_heat(k)*s% dm(k)
			if (wave_break_flag < 0.5) then
				r_dep = s% rmid(k)
				m_dep = (s% mstar - s% m(k))/Msun
				wave_break_flag = 1.
			end if
		end if

	  end do

	  s% xtra1 = L_waves_max
	  s% xtra2 = L_heat
	  s% xtra3 = conv_freq_max
	  s% xtra4 = heatfrac
	  s% xtra5 = L_conv_max
	  s% xtra6 = k_max

	  !print*, conv_freq_max
	  !print*, r_max
          !print*, Tl1,intkrl1
	  !print*, Tl1min,intkrl1min,intmin
	  !print*, Tl1b,intkrl1b
	  !print*, krl1min,kmin,bruntmin
	  !print*, s% mass_conv_core
	  print*, conv_freq_max,heatfrac,L_waves_max/Lsun,L_waves/Lsun,L_heat/Lsun

      end subroutine wave_energy









      integer function how_many_extra_profile_columns(id, id_extra)
         use star_def, only: star_info
         integer, intent(in) :: id, id_extra
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_profile_columns = 27
      end function how_many_extra_profile_columns
      
      
      subroutine data_for_extra_profile_columns(id, id_extra, n, nz, names, vals, ierr)
         use star_def, only: star_info, maxlen_profile_column_name
         use const_def, only: dp
         integer, intent(in) :: id, id_extra, n, nz
         character (len=maxlen_profile_column_name) :: names(n)
         real(dp) :: vals(nz,n)
         real(dp) :: om_con,diff,diff_t,diff_c,vg,epsheat,ebind,debind,brunts,lamb1,lamb2,lamb3
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         integer :: k
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         
         !note: do NOT add the extra names to profile_columns.list
         ! the profile_columns.list is only for the built-in profile column options.
         ! it must not include the new column names you are adding here.

         !if (n /= 1) stop 'data_for_extra_profile_columns'
         names(1) = 'L_waves'
	 names(2) = 'log_L_waves'
	 names(3) = 'conv_freq'
	 names(4) = 'log_conv_freq'
	 names(5) = 'M_damp'
	 names(6) = 'log_M_damp'
	 names(7) = 't_therm'
	 names(8) = 'log_t_therm'
	 names(9) = 't_diff'
	 names(10) = 'log_t_diff'
	 names(11) = 't_heat'
	 names(12) = 'log_t_heat'
	 names(13) = 't_cross'
	 names(14) = 'log_t_cross'
	 names(15) = 't_cool'
	 names(16) = 'log_t_cool'
	 names(17) = 't_dyn'
	 names(18) = 'log_t_dyn'	 
	 names(19) = 'log_L_conv'
	 names(20) = 'Mdot'
	 names(21) = 'Ebind'
	 names(22) = 'logEbind'
	 names(23) = 'log_N'
	 names(24) = 'log_L1'
	 names(25) = 'log_L2'
	 names(26) = 'log_L3'
	 names(27) = 'log_f_con'
	 !names(28) = 'log_alpha_RTI'

	 !names(28) = 'dlnepsnucdlnT'
	 !names(29) = 'dlnnonnucdlnT'
	 !names(30) = 'depstot'


	 ebind = 0.

         do k = 1, nz

	    if (s% brunt_N2(k) < 1d-14) brunts = 1d-14 
	    if (s% brunt_N2(k) > 1d-14) brunts = s% brunt_N2(k) 

	    !Wave generation power
            vals(k,1) = (s% conv_vel(k)/s% csound(k))*s% L_conv(k)/Lsun
	    vals(k,2) = safe_log10_cr(vals(k,1))

	    !Linear convective turnover frequency
	    vals(k,3) = (s% conv_vel(k)/(4.*s% scale_height(k)))
	    vals(k,4) = safe_log10_cr(vals(k,3))

	    !Calculate damping mass in solar masses
	    om_con = 2.*pi*s% xtra3
	    diff_t = 16.*boltz_sigma*pow3(s% T(k))/(3.*pow2(s% rho(k))*s% cp(k)*s% opacity(k))
	    diff_c = 0.!s% conv_vel(k)*s% scale_height(k)
	    diff = diff_t + diff_c
	    vals(k,5) = 4.*pi*s% rho(k)*pow2(s% rmid(k))*pow3(s% csound(k))/(pow2(om_con)*diff*Msun)
	    vals(k,6) = safe_log10_cr(vals(k,5))

	    !Thermal time scale in years
	    vals(k,7) = 4.*pi*s% rho(k)*pow2(s% rmid(k))*pow2(s% csound(k))*s% scale_height(k)/(s% L(k)*3.15d7)
	    vals(k,8) = safe_log10_cr(vals(k,7))

	    !Photon diffusion time scale in years
	    vals(k,9) = (s% r(1) - s% r(k))*s% tau(k)/(clight*3.15d7)
	    vals(k,10) = safe_log10_cr(vals(k,9))

	    !Wave heating time scale in years
	    vals(k,11) = pow2(s% csound(k))/(s% extra_heat(k)*3.15d7)
	    vals(k,12) = safe_log10_cr(vals(k,11))

	    !Gravity wave crossing time scale across a scale height, in years
	    vg = pow2(om_con)*s% r(k)/sqrt(2.*brunts)
	    vals(k,13) = s% scale_height(k)/(vg*3.15d7)
	    vals(k,14) = safe_log10_cr(vals(k,13))

	    !Neutrino cooling time scale, in years
	    epsheat = s% eps_nuc(k) - s% non_nuc_neu(k)
	    vals(k,15) = (s% cp(k)*s% T(k))/(-10.*epsheat*3.15d7)
	    vals(k,16) = safe_log10_cr(vals(k,15))

	    !Local dynamical time, in years
	    vals(k,17) = (s% scale_height(k))/(s% csound(k)*3.15d7)
	    vals(k,18) = safe_log10_cr(vals(k,17))

	    !Convective luminosity
	    vals(k,19) = safe_log10_cr(s% L_conv(k)/Lsun)

	    !Local mass outflow rate, in Msun/year
	    vals(k,20) = 4.*pi*s% rho(k)*pow2(s% r(k))*s% v(k)*(3.156d7/Msun)

	    !Calculate total binding energy of overlying envelope
	    debind = 6.67d-8*s% m(k)*s% dm(k)/s% r(k)
	    ebind = ebind + debind
	    vals(k,21) = ebind
	    vals(k,22) = safe_log10_cr(vals(k,21))

	    vals(k,23) = safe_log10_cr(sqrt(brunts)/(2.*pi))

	    lamb1 = 2.*pow2(s% csound(k)/(s% r(k)))
	    lamb2 = 6.*pow2(s% csound(k)/(s% r(k)))
	    lamb2 = 12.*pow2(s% csound(k)/(s% r(k)))

	    vals(k,24) = safe_log10_cr(sqrt(lamb1)/(2.*pi))
	    vals(k,25) = safe_log10_cr(sqrt(lamb2)/(2.*pi))
	    vals(k,26) = safe_log10_cr(sqrt(lamb3)/(2.*pi))

	    vals(k,27) = safe_log10_cr(om_con/(2.*pi))

	    !vals(k,28) = safe_log10_cr(s% alpha_RTI(k))

	    !vals(k,28) = s% d_epsnuc_dlnT(k)/(s% eps_nuc(k))
	    !vals(k,29) = s% d_nonnucneu_dlnT(k)/(s% non_nuc_neu(k))
	    !vals(k,30) = s% d_epsnuc_dlnT(k) - s% d_nonnucneu_dlnT(k)


         end do

	 !print*, ebind

      end subroutine data_for_extra_profile_columns





      integer function how_many_extra_history_columns(id, id_extra)
         integer, intent(in) :: id, id_extra
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_history_columns = 8
      end function how_many_extra_history_columns
      
      
      subroutine data_for_extra_history_columns(id, id_extra, n, names, vals, ierr)
         integer, intent(in) :: id, id_extra, n
         character (len=maxlen_history_column_name) :: names(n)
         real(dp) :: vals(n)
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return

         names(1) = 'L_waves_max'
         names(2) = 'L_heat'
	 names(3) = 'conv_freq_max'
	 names(4) = 'heatfrac'
	 names(5) = 'L_conv_max'
	 names(6) = 'k_max'
	 names(7) = 'power_nuc_burn'
	 names(8) = 'power_neutrinos'

	 vals(1) = s% xtra1
	 vals(2) = s% xtra2
	 vals(3) = s% xtra3
	 vals(4) = s% xtra4
	 vals(5) = s% xtra5
	 vals(6) = s% xtra6
	 vals(7) = s% power_nuc_burn
	 vals(8) = s% power_neutrinos

         !note: do NOT add the extras names to history_columns.list
         ! the history_columns.list is only for the built-in log column options.
         ! it must not include the new column names you are adding here.
         

      end subroutine data_for_extra_history_columns



































      
      ! None of the following functions are called unless you set their
      ! function point in extras_control.
      
      
      integer function extras_startup(id, restart, ierr)
         integer, intent(in) :: id
         logical, intent(in) :: restart
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_startup = 0
         if (.not. restart) then
            call alloc_extra_info(s)
         else ! it is a restart
            call unpack_extra_info(s)
         end if
      end function extras_startup
      

      ! returns either keep_going, retry, backup, or terminate.
      integer function extras_check_model(id, id_extra)
         integer, intent(in) :: id, id_extra
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_check_model = keep_going         
         if (.false. .and. s% star_mass_h1 < 0.35d0) then
            ! stop when star hydrogen mass drops to specified level
            extras_check_model = terminate
            write(*, *) 'have reached desired hydrogen mass'
            return
         end if


         ! if you want to check multiple conditions, it can be useful
         ! to set a different termination code depending on which
         ! condition was triggered.  MESA provides 9 customizeable
         ! termination codes, named t_xtra1 .. t_xtra9.  You can
         ! customize the messages that will be printed upon exit by
         ! setting the corresponding termination_code_str value.
         ! termination_code_str(t_xtra1) = 'my termination condition'

         ! by default, indicate where (in the code) MESA terminated
         if (extras_check_model == terminate) s% termination_code = t_extras_check_model
      end function extras_check_model


      

      ! returns either keep_going or terminate.
      ! note: cannot request retry or backup; extras_check_model can do that.
      integer function extras_finish_step(id, id_extra)
         integer, intent(in) :: id, id_extra
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_finish_step = keep_going
         call store_extra_info(s)

         ! to save a profile, 
            ! s% need_to_save_profiles_now = .true.
         ! to update the star log,
            ! s% need_to_update_history_now = .true.

         ! see extras_check_model for information about custom termination codes
         ! by default, indicate where (in the code) MESA terminated
         if (extras_finish_step == terminate) s% termination_code = t_extras_finish_step
      end function extras_finish_step
      
      
      subroutine extras_after_evolve(id, id_extra, ierr)
         integer, intent(in) :: id, id_extra
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
      end subroutine extras_after_evolve
      
      
      ! routines for saving and restoring extra data so can do restarts
         
         ! put these defs at the top and delete from the following routines
         !integer, parameter :: extra_info_alloc = 1
         !integer, parameter :: extra_info_get = 2
         !integer, parameter :: extra_info_put = 3
      
      
      subroutine alloc_extra_info(s)
         integer, parameter :: extra_info_alloc = 1
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_alloc)
      end subroutine alloc_extra_info
      
      
      subroutine unpack_extra_info(s)
         integer, parameter :: extra_info_get = 2
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_get)
      end subroutine unpack_extra_info
      
      
      subroutine store_extra_info(s)
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_put)
      end subroutine store_extra_info
      
      
      subroutine move_extra_info(s,op)
         integer, parameter :: extra_info_alloc = 1
         integer, parameter :: extra_info_get = 2
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         integer, intent(in) :: op
         
         integer :: i, j, num_ints, num_dbls, ierr
         
         i = 0
         ! call move_int or move_flg    
         num_ints = i
         
         i = 0
         ! call move_dbl       
         
         num_dbls = i
         
         if (op /= extra_info_alloc) return
         if (num_ints == 0 .and. num_dbls == 0) return
         
         ierr = 0
         call star_alloc_extras(s% id, num_ints, num_dbls, ierr)
         if (ierr /= 0) then
            write(*,*) 'failed in star_alloc_extras'
            write(*,*) 'alloc_extras num_ints', num_ints
            write(*,*) 'alloc_extras num_dbls', num_dbls
            stop 1
         end if
         
         contains
         
         subroutine move_dbl(dbl)
            real(dp) :: dbl
            i = i+1
            select case (op)
            case (extra_info_get)
               dbl = s% extra_work(i)
               !dbl = s% xtra1_array(i)
            case (extra_info_put)
               s% extra_work(i) = dbl
	       !s% xtra1_array(i) = dbl
            end select
         end subroutine move_dbl
         
         subroutine move_int(int)
            integer :: int
            i = i+1
            select case (op)
            case (extra_info_get)
               int = s% extra_iwork(i)
            case (extra_info_put)
               s% extra_iwork(i) = int
            end select
         end subroutine move_int
         
         subroutine move_flg(flg)
            logical :: flg
            i = i+1
            select case (op)
            case (extra_info_get)
               flg = (s% extra_iwork(i) /= 0)
            case (extra_info_put)
               if (flg) then
                  s% extra_iwork(i) = 1
               else
                  s% extra_iwork(i) = 0
               end if
            end select
         end subroutine move_flg
      
      end subroutine move_extra_info

      
      










      end module run_star_extras
      
